
# q-expansion of newform 4248.2.a.b, downloaded from the LMFDB on 11 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4248
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-2], [0], [4], [0], [0], [-4], [0], [-6], [-4], [8], [12], [-6], [-4], [2], [1], [8], [-14], [10], [-10], [8], [12], [-14], [-14], [14], [16], [0], [-8], [-14], [-16], [-20], [-12], [-4], [-14], [-16], [-12], [-8], [-18], [-18], [12], [-10], [8], [14], [-6], [-8], [-22], [24], [12], [-4], [14], [-22], [-26], [12], [-12], [-2], [2], [8], [18], [-8], [2], [-6], [16], [18], [-10], [-14], [0], [-26], [-4], [24], [10], [34], [28], [-2], [-16], [6], [22], [-28], [-6], [26], [36], [32], [-12], [-14], [-16], [20], [0], [34], [-2], [-4], [4], [-14], [-8], [8], [4], [-16], [2], [-4], [-44], [-16], [20], [-18], [20], [-42], [10], [-14], [-28], [-4], [-30], [26], [24], [-16], [-12], [16], [16], [-24], [44], [14], [6], [20], [38], [6], [14], [36], [26], [-30], [-10], [0], [-16], [-38], [34], [-34], [-24], [22], [-40], [30], [-18], [-32], [42], [-18], [18], [42], [-32], [-20], [-10], [44], [-6], [42], [-10], [-56], [-38], [-42], [12], [-52], [4], [6], [28], [-42], [-14], [18], [12], [8], [0], [12], [2], [-24], [48], [-38], [58], [-6], [20], [52], [18], [54], [64], [44], [-40], [6], [-8], [62], [8], [20], [40], [0], [-46], [-10], [-48], [62], [-46], [-60], [-26], [-28], [10], [-18], [60], [-6], [22], [12], [-18], [44], [54], [32], [14], [54], [40], [54], [-56], [44], [-46], [-50], [-42], [30], [-48], [-44], [38], [-2], [40], [64], [-12], [-26], [-24], [-12], [6], [-8], [-20], [40], [40], [-28], [48], [4], [-64], [-6], [68], [-18], [26], [46], [-46], [42], [4], [18], [16], [-26], [-16], [-30], [36], [42], [20], [-20], [0], [-22], [-58], [30], [-52], [-74], [10], [-68], [-34], [50], [26], [-54], [60], [-60], [10], [10], [48], [10], [54], [-18], [4], [-26], [8], [16], [-18], [8], [-22], [8], [-68], [22], [10], [-36], [66], [20], [-24], [-68], [-6], [-12], [-30], [54], [16], [0], [-6], [0], [-44], [-84], [10], [30], [48], [-14], [-36], [-54], [54], [26], [40], [-36], [-28], [-38], [60], [16], [-16], [-68], [-6], [18], [16], [-74], [62], [-34], [60], [-54], [-18], [76], [-64], [-18], [42], [-64], [56], [-22], [50], [-40], [-28], [-90], [24], [74], [28], [60], [12], [12], [50], [-62], [-42], [-80], [-58], [-50], [26], [80], [66], [60], [52], [-44], [-72], [42], [86], [26], [66], [-92], [-86], [34], [-60], [-56], [72], [78], [-32], [48], [78], [68], [-72], [-94], [30], [8], [-2], [36], [-50], [-84], [-78], [16], [42], [76], [-44], [10], [30], [-34], [-66], [46], [-88], [38], [16], [-60], [80], [22], [-16], [-92], [-66], [-90], [60], [-26], [72], [-86], [-56], [20], [76], [-90], [-70], [-90], [-32], [-32], [18], [-68], [-52], [-100], [74], [68], [-26], [38], [-68], [64], [50], [-30], [36], [-8], [-42], [-18], [26], [38], [-18], [-40], [58], [-38], [68], [48], [-4], [-38]]
